/*****************************************************************************
* voicesync_stats.c	Amfeltec VoiceSync - Debug core
*
*
* Copyright:	(c) 2010 Amfeltec Corp.
*
* ============================================================================
* June 10, 2008		Initial version.
*****************************************************************************/

/***************************************************************************
****		I N C L U D E  		F I L E S			****
***************************************************************************/


#include "os/os_abstr_includes.h"
#include "os/os_abstr_defines.h"
#include "os/os_abstr_debug.h"
#include "os/os_abstr_basic.h"
#include "os/os_abstr_time.h"

#include "voicesync_stats.h"

/***************************************************************************
****		   	  M A C R O S / D E F I N E S			****
***************************************************************************/
#define VSPCI_STATS_DELAY_MS	10000
#define VSPCI_STATS_TIMER_MS	1000

#define VSPCI_STATS_RANGE	10000
#define VSPCI_STATS_LOW		999750		// 99.9750 
#define VSPCI_TICKS_LOW		(HZ-35)

/***************************************************************************
****		     S T R U C T U R E S / T Y P E D E F S		****
***************************************************************************/

/***************************************************************************
****			F U N C T I O N  P R O T O T Y P E S		****
***************************************************************************/
static os_timer_func_t	voicesync_stats; 

/***************************************************************************
****		       G L O B A L  V A R I A B L E S			****
***************************************************************************/
static unsigned long	score_best = 0;
static unsigned long	score_worst = 100 * VSPCI_STATS_RANGE;

/***************************************************************************
****			 F U N C T I O N  D E F I N I T I O N		****
***************************************************************************/

/*===========================================================================*/
/*================= Kernel module entry functions ===========================*/
int voicesync_stats_init(voicesync_stats_t *stats)
{
	int rc = 0;

	DEBUG_EVENT("Enabling voicesync stats timer...\n");
	stats->skip = 5;

	/* Enable statistics timer */
	os_init_timer(&stats->timer, &voicesync_stats, (os_timer_arg_t)stats);
	rc = os_add_timer(&stats->timer, VSPCI_STATS_DELAY_MS);
	if (!rc){
		stats->ready = 1;
	}
	return 0;
}

void voicesync_stats_release(voicesync_stats_t *stats)
{

	if (!stats->ready) return;

	os_del_timer(&stats->timer);

	DEBUG_EVENT("VoiceSync(HW) result: Best: %ld.%ld%% Worst: %ld.%ld%%\n",
		score_best / VSPCI_STATS_RANGE, score_best % VSPCI_STATS_RANGE, 
		score_worst / VSPCI_STATS_RANGE, score_worst % VSPCI_STATS_RANGE); 
	return;
}

#if defined(__NetBSD__) || defined(__FreeBSD__) || defined(__OpenBSD__)
static void voicesync_stats(void* arg1)
#elif defined(__WINDOWS__)
static void voicesync_stats(IN PKDPC Dpc, void* arg1, void* arg2, void* arg3)
#else
static void voicesync_stats(unsigned long arg1)
#endif
{
	voicesync_stats_t 	*stats = (voicesync_stats_t*)arg1;
	int			rxbytes;
	unsigned long		score, percent;
	os_time_t 		sec;
	os_suseconds_t		usec;

	OS_ASSERT_VOID(stats == NULL);

	if (stats->count == 0) stats->ms = 0;

	rxbytes = stats->rx_bytes;
	os_getcurrenttime(&sec, &usec);

	stats->count += (rxbytes - stats->last_rxbytes);
	stats->last_rxbytes = rxbytes;
	stats->ms += (sec - stats->last_sec) * 8000;
	stats->ms += (usec - stats->last_usec) / 125; 
	
	if (stats->count >= 8000){

		percent = (100 * VSPCI_STATS_RANGE * abs(stats->count - stats->ms)) / stats->count;
		score = 100 * VSPCI_STATS_RANGE - abs(percent);

		if (!stats->skip){
			if (score > score_best) score_best = score;
			if (score < score_worst) score_worst = score;
			if (score < VSPCI_STATS_LOW){
#if VS_VERBOSE >= 2
				DEBUG_EVENT("VoiceSync stats: %ld.%ld%%\n",
						score / VSPCI_STATS_RANGE, 
						score % VSPCI_STATS_RANGE);
#endif
			}
		}else stats->skip--;
		stats->count = 0;
	}
	stats->last_sec = sec;
	stats->last_usec = usec;

	os_add_timer(&stats->timer, VSPCI_STATS_TIMER_MS);
	return;
}

